@@ -81,7 +81,6 @@ import base64
 import hashlib
 import struct
 import json
-import collections
 import tempfile
 import time
 import traceback
@@ -111,7 +110,7 @@ DEBUG = 0 if os.getenv('NODEBUG') else int(os.getenv('DEBUG', 1))
 class Websocket: # {{{
 	'''Main class implementing the websocket protocol.
 	'''
-	def __init__(self, port, url = '/', recv = None, method = 'GET', user = None, password = None, extra = {}, socket = None, mask = (None, True), websockets = None, data = None, real_remote = None, *a, **ka): # {{{
+	def __init__(self, port, recv = None, method = 'GET', user = None, password = None, extra = {}, socket = None, mask = (None, True), websockets = None, data = None, real_remote = None, *a, **ka): # {{{
 		'''When constructing a Websocket, a connection is made to the
 		requested port, and the websocket handshake is performed.  This
 		constructor passes any extra arguments to the network.Socket
@@ -121,8 +120,7 @@ class Websocket: # {{{
 		some arguments that should not be used when calling it
 		directly.
 		@param port: Host and port to connect to, same format as
-			python-network uses.
-		@param url: The url to request at the host.
+			python-network uses, or None for an incoming connection (internally used).
 		@param recv: Function to call when a data packet is received
 			asynchronously.
 		@param method: Connection method to use.
@@ -168,7 +166,7 @@ class Websocket: # {{{
 		else:
 			self.remote = socket.remote
 		hdrdata = b''
-		if url is not None:
+		if port is not None:
 			elist = []
 			for e in extra:
 				elist.append('%s: %s\r\n' % (e, extra[e]))
@@ -176,6 +174,16 @@ class Websocket: # {{{
 				userpwd = user + ':' + password + '\r\n'
 			else:
 				userpwd = ''
+			p = re.match('^(?:([a-z0-9-]+)://)?([^:/?#]+)(?::([^:/?#]+))?([:/?#].*)?$', port)
+			# Group 1: protocol or None
+			# Group 2: hostname
+			# Group 3: port
+			# Group 4: everything after the port (address, query string, etc)
+			url = p.group(4)
+			if url is None:
+				url = '/'
+			elif not url.startswith('/'):
+				url = '/' + url
 			socket.send(('''\
 %s %s HTTP/1.1\r
 Connection: Upgrade\r
@@ -212,12 +220,12 @@ Sec-WebSocket-Key: 0\r
 				self._is_closed = True
 				if self.websockets is not None:
 					self.websockets.remove(self)
-				self.closed()
+				self._websocket_closed()
 			return b''
 		if self.websockets is not None:
 			self.websockets.add(self)
 		self.socket.disconnect_cb(disconnect)
-		self.opened()
+		self._websocket_opened()
 		if len(hdrdata) > 0:
 			self._websocket_read(hdrdata)
 		if DEBUG > 2:
@@ -339,11 +347,11 @@ Sec-WebSocket-Key: 0\r
 			self.opcode = None
 			if opcode == 8:
 				# Connection close request.
-				self.close()
+				self._websocket_close()
 				return None
 			elif opcode == 9:
 				# Ping.
-				self.send(data, 10)	# Pong
+				self._websocket_send(data, 10)	# Pong
 			elif opcode == 10:
 				# Pong.
 				self._pong = True
@@ -368,7 +376,7 @@ Sec-WebSocket-Key: 0\r
 				log('invalid opcode')
 				self.socket.close()
 	# }}}
-	def send(self, data, opcode = 1):	# Send a WebSocket frame.  {{{
+	def _websocket_send(self, data, opcode = 1):	# Send a WebSocket frame.  {{{
 		'''Send a Websocket frame to the remote end of the connection.
 		@param data: Data to send.
 		@param opcode: Opcade to send.  0 = fragment, 1 = text packet, 2 = binary packet, 8 = close request, 9 = ping, 10 = pong.
@@ -404,24 +412,24 @@ Sec-WebSocket-Key: 0\r
 		if opcode == 8:
 			self.socket.close()
 	# }}}
-	def ping(self, data = b''): # Send a ping; return False if no pong was seen for previous ping.  Other received packets also count as a pong. {{{
+	def _websocket_ping(self, data = b''): # Send a ping; return False if no pong was seen for previous ping.  Other received packets also count as a pong. {{{
 		'''Send a ping, return if a pong was received since last ping.
 		@param data: Data to send with the ping.
 		@return True if a pong was received since last ping, False if not.
 		'''
 		ret = self._pong
 		self._pong = False
-		self.send(data, opcode = 9)
+		self._websocket_send(data, opcode = 9)
 		return ret
 	# }}}
-	def close(self):	# Close a WebSocket.  (Use self.socket.close for other connections.)  {{{
+	def _websocket_close(self):	# Close a WebSocket.  (Use self.socket.close for other connections.)  {{{
 		'''Send close request, and close the connection.
 		@return None.
 		'''
-		self.send(b'', 8)
+		self._websocket_send(b'', 8)
 		self.socket.close()
 	# }}}
-	def opened(self): # {{{
+	def _websocket_opened(self): # {{{
 		'''This function does nothing by default, but can be overridden
 		by the application.  It is called when a new websocket is
 		opened.  As this happens from the constructor, it is useless to
@@ -431,7 +439,7 @@ Sec-WebSocket-Key: 0\r
 		'''
 		pass
 	# }}}
-	def closed(self): # {{{
+	def _websocket_closed(self): # {{{
 		'''This function does nothing by default, but can be overridden
 		by the application.  It is called when the websocket is closed.
 		@return None.
@@ -529,7 +537,7 @@ class RPC(Websocket): # {{{
 		calls = self._delayed_calls
 		self._delayed_calls = None
 		for call in calls:
-			if not hasattr(self._target, call[1]) or not isinstance(getattr(self._target, call[1]), collections.Callable):
+			if not hasattr(self._target, call[1]) or not callable(getattr(self._target, call[1])):
 				self._send('error', 'invalid delayed call frame %s' % repr(call))
 			else:
 				self._call(call[0], call[1], call[2], call[3])
@@ -560,6 +568,10 @@ class RPC(Websocket): # {{{
 			self.base._send('call', (my_id, self.attr, a, ka))
 			RPC._calls[my_id] = lambda x: self.do_reply(reply, my_id, x)
 		# }}}
+		def wait(self, wake, *a, **ka): # {{{
+			self.bg(wake, *a, **ka)
+			return (yield)
+		# }}}
 		def do_reply(self, reply, my_id, ret): # {{{
 			del RPC._calls[my_id]
 			reply(ret)
@@ -580,7 +592,7 @@ class RPC(Websocket): # {{{
 		'''
 		if DEBUG > 1:
 			log('sending:' + repr(type) + repr(object))
-		Websocket.send(self, json.dumps((type, object)))
+		Websocket._websocket_send(self, json.dumps((type, object)))
 	# }}}
 	def _parse_frame(self, frame): # {{{
 		'''Decode an RPC packet.
@@ -615,7 +627,7 @@ class RPC(Websocket): # {{{
 			if not isinstance(data[1][3], dict):
 				log('invalid call frame (no dict kwargs) %s' % repr(data))
 				return (None, 'invalid frame')
-			if (self._delayed_calls is None and (not hasattr(self._target, data[1][1]) or not isinstance(getattr(self._target, data[1][1]), collections.Callable))):
+			if self._delayed_calls is None and (not hasattr(self._target, data[1][1]) or not callable(getattr(self._target, data[1][1]))):
 				log('invalid call frame (no callable) %s' % repr(data))
 				return (None, 'invalid frame')
 		elif data[0] not in ('error', 'return'):
@@ -829,7 +841,7 @@ class _Httpd_connection:	# {{{
 		newkey = base64.b64encode(hashlib.sha1(self.headers['sec-websocket-key'].strip().encode('utf-8') + b'258EAFA5-E914-47DA-95CA-C5AB0DC85B11').digest()).decode('utf-8')
 		headers = {'Sec-WebSocket-Accept': newkey, 'Connection': 'Upgrade', 'Upgrade': 'websocket', 'Sec-WebSocket-Version': '13'}
 		self.server.reply(self, 101, None, None, headers, close = False)
-		self.websocket(None, recv = self.server.recv, url = None, socket = self.socket, error = self.error, mask = (None, False), websockets = self.server.websockets, data = self.data, real_remote = self.headers.get('x-forwarded-for'))
+		self.websocket(None, recv = self.server.recv, socket = self.socket, error = self.error, mask = (None, False), websockets = self.server.websockets, data = self.data, real_remote = self.headers.get('x-forwarded-for'))
 	# }}}
 	def _parse_headers(self, message): # {{{
 		lines = []
@@ -1079,6 +1091,8 @@ class Httpd: # {{{
 			exts = {}
 			with open('/etc/mime.types') as f:
 				for ln in f:
+					if ln.strip() == '' or ln.startswith('#'):
+						continue
 					items = ln.split()
 					for ext in items[1:]:
 						if ext not in exts:
